@@ -139,7 +139,11 @@ unbold_timeout_cb (gpointer user_data)
   PangoFontDescription *desc;
 
   desc = pango_font_description_new ();
+#if GTK_CHECK_VERSION(3,0,0)
+  gtk_widget_override_font (findbar->priv->message, desc);
+#else
   gtk_widget_modify_font (findbar->priv->message, desc);
+#endif
   pango_font_description_free (desc);
 
   findbar->priv->status_bold_id = 0;
@@ -188,7 +192,11 @@ logview_findbar_init (LogviewFindbar *findbar)
   gtk_widget_show_all (GTK_WIDGET (item));
 
   /* "Previous" and "Next" buttons */
+#if GTK_CHECK_VERSION (3, 0, 0)
+  w = gtk_image_new_from_icon_name ("pan-start-symbolic", GTK_ICON_SIZE_BUTTON);
+#else
   w = gtk_arrow_new (GTK_ARROW_LEFT, GTK_SHADOW_NONE);
+#endif
   priv->back_button = gtk_tool_button_new (w, _("Find Previous"));
   gtk_tool_item_set_is_important (priv->back_button, TRUE);
   gtk_tool_item_set_tooltip_text (priv->back_button,
@@ -196,7 +204,11 @@ logview_findbar_init (LogviewFindbar *findbar)
   gtk_toolbar_insert (gtoolbar, priv->back_button, -1);
   gtk_widget_show_all (GTK_WIDGET (priv->back_button));
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+  w = gtk_image_new_from_icon_name ("pan-end-symbolic", GTK_ICON_SIZE_BUTTON);
+#else
   w = gtk_arrow_new (GTK_ARROW_RIGHT, GTK_SHADOW_NONE);
+#endif
   priv->forward_button = gtk_tool_button_new (w, _("Find Next"));
   gtk_tool_item_set_is_important (priv->forward_button, TRUE);
   gtk_tool_item_set_tooltip_text (priv->forward_button,
@@ -220,7 +232,12 @@ logview_findbar_init (LogviewFindbar *findbar)
   gtk_tool_item_set_expand (priv->status_item, TRUE);
   priv->message = gtk_label_new ("");
   gtk_label_set_use_markup (GTK_LABEL (priv->message), TRUE);
+#if GTK_CHECK_VERSION(3,16,0)
+  gtk_label_set_xalign (GTK_LABEL (priv->message), 0.0);
+  gtk_label_set_yalign (GTK_LABEL (priv->message), 0.5);
+#else
   gtk_misc_set_alignment (GTK_MISC (priv->message), 0.0, 0.5);
+#endif
   gtk_container_add (GTK_CONTAINER (priv->status_item), priv->message);
   gtk_widget_show (priv->message);
   gtk_toolbar_insert (gtoolbar, priv->status_item, -1);
@@ -343,7 +360,11 @@ logview_findbar_set_message (LogviewFindbar *findbar,
   if (text) {
     desc = pango_font_description_new ();
     pango_font_description_set_weight (desc, PANGO_WEIGHT_BOLD);
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_widget_override_font (findbar->priv->message, desc);
+#else
     gtk_widget_modify_font (findbar->priv->message, desc);
+#endif
     pango_font_description_free (desc);
     
     findbar->priv->status_bold_id = g_timeout_add (600, unbold_timeout_cb, findbar);
