@@ -315,7 +315,9 @@ create_effects_frame (GtkWidget   *outer_vbox,
                       const gchar *frame_title)
 {
   GtkWidget *main_vbox, *vbox, *hbox;
+#if !GTK_CHECK_VERSION (3, 0, 0)
   GtkWidget *align;
+#endif
   GtkWidget *label;
   GtkWidget *check;
   GtkWidget *combo;
@@ -352,17 +354,22 @@ create_effects_frame (GtkWidget   *outer_vbox,
   gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
+#if !GTK_CHECK_VERSION (3, 0, 0)
   align = gtk_alignment_new (0.0, 0.0, 0.0, 0.0);
   gtk_alignment_set_padding (GTK_ALIGNMENT (align), 0, 0, 12, 0);
   gtk_box_pack_start (GTK_BOX (hbox), align, FALSE, FALSE, 0);
   gtk_widget_show (align);
+#endif
 
 #if GTK_CHECK_VERSION (3, 0, 0)
   vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 0);
+  gtk_widget_set_margin_start (vbox, 12);
 #else
   vbox = gtk_vbox_new (FALSE, 6);
-#endif
   gtk_container_add (GTK_CONTAINER (align), vbox);
+#endif
+
   gtk_widget_show (vbox);
 
   /** Include pointer **/
@@ -419,7 +426,9 @@ create_screenshot_frame (GtkWidget   *outer_vbox,
                          const gchar *frame_title)
 {
   GtkWidget *main_vbox, *vbox, *hbox;
+#if !GTK_CHECK_VERSION (3, 0, 0)
   GtkWidget *align;
+#endif
   GtkWidget *radio;
   GtkWidget *image;
   GtkWidget *spin;
@@ -456,15 +465,25 @@ create_screenshot_frame (GtkWidget   *outer_vbox,
 #endif
   gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
-
+#if !GTK_CHECK_VERSION (3, 0, 0)
   align = gtk_alignment_new (0.0, 0.0, 0.0, 0.0);
   gtk_widget_set_size_request (align, 48, -1);
   gtk_box_pack_start (GTK_BOX (hbox), align, FALSE, FALSE, 0);
   gtk_widget_show (align);
+#endif
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+  image = gtk_image_new_from_icon_name (SCREENSHOOTER_ICON,
+                                    GTK_ICON_SIZE_DIALOG);
+
+  gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+  gtk_widget_set_valign (image, GTK_ALIGN_START);
+#else
   image = gtk_image_new_from_stock (SCREENSHOOTER_ICON,
                                     GTK_ICON_SIZE_DIALOG);
   gtk_container_add (GTK_CONTAINER (align), image);
+#endif
+
   gtk_widget_show (image);
 
 #if GTK_CHECK_VERSION (3, 0, 0)
@@ -586,13 +605,11 @@ create_interactive_dialog (void)
 
   create_screenshot_frame (main_vbox, _("Take Screenshot"));
   create_effects_frame (main_vbox, _("Effects"));
-
   gtk_dialog_add_buttons (GTK_DIALOG (retval),
                           GTK_STOCK_HELP, GTK_RESPONSE_HELP,
                           GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                           _("Take _Screenshot"), GTK_RESPONSE_OK,
                           NULL);
-
   gtk_dialog_set_default_response (GTK_DIALOG (retval), GTK_RESPONSE_OK);
 
   /* we need to block on "response" and keep showing the interactive
@@ -1315,6 +1332,7 @@ main (int argc, char *argv[])
   gboolean disable_border_arg = FALSE;
   gboolean interactive_arg = FALSE;
   gchar *border_effect_arg = NULL;
+  gboolean version_arg = FALSE;
   guint delay_arg = 0;
   GError *error = NULL;
 
@@ -1326,6 +1344,7 @@ main (int argc, char *argv[])
     { "delay", 'd', 0, G_OPTION_ARG_INT, &delay_arg, N_("Take screenshot after specified delay [in seconds]"), N_("seconds") },
     { "border-effect", 'e', 0, G_OPTION_ARG_STRING, &border_effect_arg, N_("Effect to add to the border (shadow, border or none)"), N_("effect") },
     { "interactive", 'i', 0, G_OPTION_ARG_NONE, &interactive_arg, N_("Interactively set options"), NULL },
+    { "version", 0, 0, G_OPTION_ARG_NONE, &version_arg, N_("Print version information and exit"), NULL },
     { NULL },
   };
 
@@ -1350,6 +1369,11 @@ main (int argc, char *argv[])
   }
 
   g_option_context_free (context);
+ 
+  if (version_arg) {
+    g_print ("%s %s\n", g_get_application_name (), VERSION);
+    exit (EXIT_SUCCESS);
+  }
 
   if (window_arg && area_arg) {
     g_printerr (_("Conflicting options: --window and --area should not be "
