@@ -240,14 +240,26 @@ GdkWindow *
 screenshot_find_current_window ()
 {
   GdkWindow *current_window;
+#if GTK_CHECK_VERSION (3, 0, 0)
+  GdkDeviceManager *manager;
+  GdkDevice *device;
+#endif
 
   current_window = screenshot_find_active_window ();
+#if GTK_CHECK_VERSION (3, 0, 0)
+  manager = gdk_display_get_device_manager (gdk_display_get_default ());
+  device = gdk_device_manager_get_client_pointer (manager);
+#endif
 
   /* If there's no active window, we fall back to returning the
    * window that the cursor is in.
    */
   if (!current_window)
+#if GTK_CHECK_VERSION (3, 0, 0)
+    current_window = gdk_device_get_window_at_position (device, NULL, NULL);
+#else
     current_window = gdk_window_at_pointer (NULL, NULL);
+#endif
 
   if (current_window)
     {
@@ -401,28 +413,49 @@ draw (GtkWidget *window, cairo_t *cr, gpointer unused)
 expose (GtkWidget *window, GdkEventExpose *event, gpointer unused)
 #endif
 {
+#if GTK_CHECK_VERSION (3, 0, 0)
+  GtkStyleContext *style;
+
+  style = gtk_widget_get_style_context (window);
+#else
   GtkAllocation allocation;
   GtkStyle *style;
-#if !GTK_CHECK_VERSION (3, 0, 0)
   cairo_t *cr;
 
   cr = gdk_cairo_create (event->window);
   gdk_cairo_region (cr, event->region);
   cairo_clip (cr);
-#endif
 
   style = gtk_widget_get_style (window);
+#endif
 
   if (gtk_widget_get_app_paintable (window))
     {
+#if !GTK_CHECK_VERSION (3, 0, 0)
       cairo_set_line_width (cr, 1.0);
 
       gtk_widget_get_allocation (window, &allocation);
+#endif
 
       cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
       cairo_set_source_rgba (cr, 0, 0, 0, 0);
       cairo_paint (cr);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+      gtk_style_context_save (style);
+      gtk_style_context_add_class (style, GTK_STYLE_CLASS_RUBBERBAND);
+
+      gtk_render_background (style, cr,
+                             0, 0,
+                             gtk_widget_get_allocated_width (window),
+                             gtk_widget_get_allocated_height (window));
+      gtk_render_frame (style, cr,
+                        0, 0,
+                        gtk_widget_get_allocated_width (window),
+                        gtk_widget_get_allocated_height (window));
+
+      gtk_style_context_restore (style);
+#else
       cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
       gdk_cairo_set_source_color (cr, &style->base[GTK_STATE_SELECTED]);
       cairo_paint_with_alpha (cr, 0.25);
@@ -436,6 +469,7 @@ expose (GtkWidget *window, GdkEventExpose *event, gpointer unused)
     {
       gdk_cairo_set_source_color (cr, &style->base[GTK_STATE_SELECTED]);
       cairo_paint (cr);
+#endif
     }
 
 #if !GTK_CHECK_VERSION (3, 0, 0)
@@ -499,7 +533,13 @@ emit_select_callback_in_idle (gpointer user_data)
 void
 screenshot_select_area_async (SelectAreaCallback callback)
 {
+  GdkDisplay *display;
   GdkCursor               *cursor;
+#if GTK_CHECK_VERSION (3, 0, 0)
+  GdkDeviceManager *manager;
+  GdkDevice *pointer, *keyboard;
+  GdkGrabStatus res;
+#endif
   select_area_filter_data  data;
   GdkRectangle *rectangle;
   CallbackData *cb_data;
@@ -519,8 +559,42 @@ screenshot_select_area_async (SelectAreaCallback callback)
   g_signal_connect (data.window, "button-release-event", G_CALLBACK (select_area_button_release), &data);
   g_signal_connect (data.window, "motion-notify-event", G_CALLBACK (select_area_motion_notify), &data);
 
-  cursor = gdk_cursor_new (GDK_CROSSHAIR);
+  display = gdk_display_get_default ();
+  cursor = gdk_cursor_new_for_display (display, GDK_CROSSHAIR);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+  manager = gdk_display_get_device_manager (display);
+  pointer = gdk_device_manager_get_client_pointer (manager);
+  keyboard = gdk_device_get_associated_device (pointer);
+
+
+  res = gdk_device_grab (pointer, gtk_widget_get_window (data.window),
+                         GDK_OWNERSHIP_NONE, FALSE,
+                         GDK_POINTER_MOTION_MASK |
+                         GDK_BUTTON_PRESS_MASK | 
+                         GDK_BUTTON_RELEASE_MASK,
+                         cursor, GDK_CURRENT_TIME);
+
+  if (res != GDK_GRAB_SUCCESS)
+    {
+      g_object_unref (cursor);
+      goto out;
+    }
+
+  res = gdk_device_grab (keyboard, gtk_widget_get_window (data.window),
+                         GDK_OWNERSHIP_NONE, FALSE,
+                         GDK_KEY_PRESS_MASK |
+                         GDK_KEY_RELEASE_MASK,
+                         NULL, GDK_CURRENT_TIME);
+  if (res != GDK_GRAB_SUCCESS)
+    {
+      gdk_device_ungrab (pointer, GDK_CURRENT_TIME);
+      g_object_unref (cursor);
+      goto out;
+    }
+
+#else
+  
   if (gdk_pointer_grab (gtk_widget_get_window (data.window), FALSE,
                         GDK_POINTER_MOTION_MASK|GDK_BUTTON_PRESS_MASK|GDK_BUTTON_RELEASE_MASK,
                         NULL, cursor,
@@ -537,14 +611,24 @@ screenshot_select_area_async (SelectAreaCallback callback)
       goto out;
     }
 
+#endif
+
   gtk_main ();
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+
+  gdk_device_ungrab (keyboard, GDK_CURRENT_TIME);
+  gdk_device_ungrab (pointer, GDK_CURRENT_TIME);
+
+#else
+
   gdk_keyboard_ungrab (GDK_CURRENT_TIME);
   gdk_pointer_ungrab (GDK_CURRENT_TIME);
 
+#endif  
+
   gtk_widget_destroy (data.window);
   gdk_cursor_unref (cursor);
-
   gdk_flush ();
 
  out:
@@ -935,10 +1019,20 @@ screenshot_get_pixbuf (GdkWindow    *window,
 
       if (cursor_pixbuf != NULL)
         {
+#if GTK_CHECK_VERSION (3, 0, 0)
+          GdkDeviceManager *manager;
+          GdkDevice *device;
+#endif
           GdkRectangle r1, r2;
           gint cx, cy, xhot, yhot;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+          manager = gdk_display_get_device_manager (gdk_display_get_default ());
+          device = gdk_device_manager_get_client_pointer (manager);
+          gdk_window_get_device_position (window, device, &cx, &cy, NULL);
+#else
           gdk_window_get_pointer (window, &cx, &cy, NULL);
+#endif
           sscanf (gdk_pixbuf_get_option (cursor_pixbuf, "x_hot"), "%d", &xhot);
           sscanf (gdk_pixbuf_get_option (cursor_pixbuf, "y_hot"), "%d", &yhot);
 
